@@ -1,7 +1,7 @@
 /*
  * program that handles the dmalloc variables.
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,9 +16,9 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $
+ * $Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $
  */
 
 /*
@@ -58,11 +58,11 @@
 #include "version.h"
 
 #if INCLUDE_RCS_IDS
-#ifdef __GNUC__
-#ident "$Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $";
+#if IDENT_WORKS
+#ident "$Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $"
 #else
 static	char	*rcs_id =
-  "$Id: dmalloc.c,v 1.88 1999/05/06 15:11:46 gray Exp $";
+  "$Id: dmalloc.c,v 1.99 2000/11/13 15:47:17 gray Exp $";
 #endif
 #endif
 
@@ -72,9 +72,14 @@ static	char	*rcs_id =
 #define TOKENIZE_EQUALS	" \t="			/* for tag lines */
 #define TOKENIZE_CHARS	" \t,"			/* for tag lines */
 
-#define NO_VALUE		(-1)		/* no value ... value */
-#define INTERVAL_NO_VALUE	0		/* no value ... value */
-#define TOKENS_PER_LINE		5		/* num debug toks per line */
+#define DEBUG_ARG		'd'		/* debug argument */
+#define INTERVAL_ARG		'i'		/* interval argument */
+#define THREAD_LOCK_ON_ARG	'o'		/* lock-on argument */
+#define LINE_WIDTH		75		/* num debug toks per line */
+
+#define FILE_NOT_FOUND		1
+#define FILE_FOUND		2
+#define TOKEN_FOUND		3
 
 /*
  * default flag information
@@ -85,9 +90,9 @@ typedef struct {
 } default_t;
 
 #define RUNTIME_FLAGS	(DEBUG_LOG_STATS | DEBUG_LOG_NONFREE | \
-			 DEBUG_LOG_BAD_SPACE | DEBUG_LOG_UNKNOWN | \
+			 DEBUG_LOG_BAD_SPACE | \
 			 DEBUG_CHECK_FENCE | \
-			 DEBUG_CATCH_NULL | DEBUG_ALLOW_NONLINEAR)
+			 DEBUG_CATCH_NULL)
 #define LOW_FLAGS	(RUNTIME_FLAGS | \
 			 DEBUG_LOG_ELAPSED_TIME | \
 			 DEBUG_FREE_BLANK | DEBUG_ERROR_ABORT | \
@@ -104,6 +109,7 @@ typedef struct {
 /* NOTE: print-error is not in this list because it is special */
 
 static	default_t	defaults[] = {
+  { "none",		0 },
   { "runtime",		RUNTIME_FLAGS },
   { "run",		RUNTIME_FLAGS },
   { "low",		LOW_FLAGS },
@@ -115,85 +121,88 @@ static	default_t	defaults[] = {
 };
 
 /* argument variables */
-static	char	*address	= NULL;		/* for ADDRESS */
-static	char	bourne_b	= FALSE;	/* set bourne shell output */
-static	char	cshell_b	= FALSE;	/* set c-shell output */
-static	char	clear_b		= FALSE;	/* clear variables */
-static	int	debug		= NO_VALUE;	/* for DEBUG */
-static	int	errno_to_print	= NO_VALUE;	/* to print the error string */
-static	char	gdb_b		= FALSE;	/* set gdb output */
-static	char	*inpath		= NULL;		/* for config-file path */
-static	unsigned long interval	= INTERVAL_NO_VALUE; /* for setting INTERVAL */
-static	int	thread_lock_on	= NO_VALUE;	/* for setting LOCK_ON */
-static	char	keep_b		= FALSE;	/* keep settings override -r */
-static	char	list_tags_b	= FALSE;	/* list rc tags */
-static	char	debug_tokens_b	= FALSE;	/* list debug tokens */
-static	char	*logpath	= NULL;		/* for LOGFILE setting */
-static	char	long_tokens_b	= FALSE;	/* long-tok output */
+static	char	*address = NULL;		/* for ADDRESS */
+static	int	bourne_b = 0;			/* set bourne shell output */
+static	int	cshell_b = 0;			/* set c-shell output */
+static	int	clear_b = 0;			/* clear variables */
+static	int	debug = 0;			/* for DEBUG */
+static	int	errno_to_print = 0;		/* to print the error string */
+static	int	gdb_b = 0;			/* set gdb output */
+static	int	help_b = 0;			/* print help message */
+static	char	*inpath = NULL;			/* for config-file path */
+static	unsigned long interval = 0;		/* for setting INTERVAL */
+static	int	thread_lock_on = 0;		/* for setting LOCK_ON */
+static	int	keep_b = 0;			/* keep settings override -r */
+static	int	list_tags_b = 0;		/* list rc tags */
+static	int	debug_tokens_b = 0;		/* list debug tokens */
+static	char	*logpath = NULL;		/* for LOGFILE setting */
+static	int	long_tokens_b = 0;		/* long-tok output */
 static	argv_array_t	minus;			/* tokens to remove */
-static	char	make_changes_b	= TRUE;		/* make no changes to env */
+static	int	make_changes_b = 1;		/* make no changes to env */
 static	argv_array_t	plus;			/* tokens to add */
-static	char	remove_auto_b	= FALSE;	/* auto-remove settings */
-static	char	short_tokens_b	= FALSE;	/* short-tok output */
-static	char	*start		= NULL;		/* for START settings */
-static	char	verbose_b	= FALSE;	/* verbose flag */
-static	char	very_verbose_b	= FALSE;	/* very-verbose flag */
-static	char	*tag		= NULL;		/* maybe a tag argument */
+static	int	remove_auto_b = 0;		/* auto-remove settings */
+static	int	short_tokens_b = 0;		/* short-tok output */
+static	char	*start = NULL;			/* for START settings */
+static	int	verbose_b = 0;			/* verbose flag */
+static	int	very_verbose_b = 0;		/* very-verbose flag */
+static	char	*tag = NULL;			/* maybe a tag argument */
 
 static	argv_t	args[] = {
-  { 'b',	"bourne",	ARGV_BOOL,	&bourne_b,
-      NULL,			"set output for bourne shells" },
+  { 'b',	"bourne",	ARGV_BOOL_INT,	&bourne_b,
+    NULL,			"set output for bourne shells" },
   { ARGV_OR },
-  { 'C',	"c-shell",	ARGV_BOOL,	&cshell_b,
-      NULL,			"set output for C-type shells" },
+  { 'C',	"c-shell",	ARGV_BOOL_INT,	&cshell_b,
+    NULL,			"set output for C-type shells" },
   { ARGV_OR },
-  { 'g',	"gdb",		ARGV_BOOL,	&gdb_b,
-      NULL,			"set output for gdb parsing" },
+  { 'g',	"gdb",		ARGV_BOOL_INT,	&gdb_b,
+    NULL,			"set output for gdb parsing" },
   
-  { 'L',	"long-tokens",	ARGV_BOOL,	&long_tokens_b,
-      NULL,			"output long-tokens not 0x..." },
+  { 'L',	"long-tokens",	ARGV_BOOL_INT,	&long_tokens_b,
+    NULL,			"output long-tokens not 0x..." },
   { ARGV_OR },
-  { 'S',	"short-tokens",	ARGV_BOOL,	&short_tokens_b,
-      NULL,			"output short-tokens not 0x..." },
+  { 'S',	"short-tokens",	ARGV_BOOL_INT,	&short_tokens_b,
+    NULL,			"output short-tokens not 0x..." },
   
   { 'a',	"address",	ARGV_CHAR_P,	&address,
-      "address:#",		"stop when malloc sees address" },
-  { 'c',	"clear",	ARGV_BOOL,	&clear_b,
-      NULL,			"clear all variables not set" },
-  { 'd',	"debug-mask",	ARGV_HEX,	&debug,
-      "value",			"hex flag to set debug mask" },
-  { 'D',	"debug-tokens",	ARGV_BOOL,	&debug_tokens_b,
-      NULL,			"list debug tokens" },
+    "address:#",		"stop when malloc sees address" },
+  { 'c',	"clear",	ARGV_BOOL_INT,	&clear_b,
+    NULL,			"clear all variables not set" },
+  { DEBUG_ARG,	"debug-mask",	ARGV_HEX,	&debug,
+    "value",			"hex flag to set debug mask" },
+  { 'D',	"debug-tokens",	ARGV_BOOL_INT,	&debug_tokens_b,
+    NULL,			"list debug tokens" },
   { 'e',	"errno",	ARGV_INT,	&errno_to_print,
-      "errno",			"print error string for errno" },
+    "errno",			"print error string for errno" },
   { 'f',	"file",		ARGV_CHAR_P,	&inpath,
-      "path",			"config if not ~/.mallocrc" },
-  { 'i',	"interval",	ARGV_U_LONG,	&interval,
-      "value",			"check heap every number times" },
-  { 'k',	"keep",		ARGV_BOOL,	&keep_b,
-      NULL,			"keep settings (override -r)" },
+    "path",			"config if not ~/.mallocrc" },
+  { 'h',	"help",		ARGV_BOOL_INT,	&help_b,
+    NULL,			"print help message" },
+  { INTERVAL_ARG, "interval",	ARGV_U_LONG,	&interval,
+    "value",			"check heap every number times" },
+  { 'k',	"keep",		ARGV_BOOL_INT,	&keep_b,
+    NULL,			"keep settings (override -r)" },
   { 'l',	"logfile",	ARGV_CHAR_P,	&logpath,
-      "path",			"file to log messages to" },
+    "path",			"file to log messages to" },
   { 'm',	"minus",	ARGV_CHAR_P | ARGV_FLAG_ARRAY,	&minus,
-      "token(s)",		"del tokens from current debug" },
+    "token(s)",			"del tokens from current debug" },
   { 'n',	"no-changes",	ARGV_BOOL_NEG,	&make_changes_b,
-      NULL,			"make no changes to the env" },
-  { 'o',	"lock-on",	ARGV_INT,	&thread_lock_on,
-      "number",			"number of times to not lock" },
+    NULL,			"make no changes to the env" },
+  { THREAD_LOCK_ON_ARG, "lock-on", ARGV_INT,	&thread_lock_on,
+    "number",			"number of times to not lock" },
   { 'p',	"plus",		ARGV_CHAR_P | ARGV_FLAG_ARRAY,	&plus,
-      "token(s)",		"add tokens to current debug" },
-  { 'r',	"remove",	ARGV_BOOL,	&remove_auto_b,
-      NULL,			"remove other settings if tag" },
+    "token(s)",			"add tokens to current debug" },
+  { 'r',	"remove",	ARGV_BOOL_INT,	&remove_auto_b,
+    NULL,			"remove other settings if tag" },
   { 's',	"start",	ARGV_CHAR_P,	&start,
-      "file:line",		"start check heap after this" },
-  { 't',	"list-tags",	ARGV_BOOL,	&list_tags_b,
-      NULL,			"list tags in rc file" },
-  { 'v',	"verbose",	ARGV_BOOL,	&verbose_b,
-      NULL,			"turn on verbose output" },
-  { 'V',	"very-verbose",	ARGV_BOOL,	&very_verbose_b,
-      NULL,			"turn on very-verbose output" },
+    "file:line",		"start check heap after this" },
+  { 't',	"list-tags",	ARGV_BOOL_INT,	&list_tags_b,
+    NULL,			"list tags in rc file" },
+  { 'v',	"verbose",	ARGV_BOOL_INT,	&verbose_b,
+    NULL,			"turn on verbose output" },
+  { 'V',	"very-verbose",	ARGV_BOOL_INT,	&very_verbose_b,
+    NULL,			"turn on very-verbose output" },
   { ARGV_MAYBE,	NULL,		ARGV_CHAR_P,	&tag,
-      "tag",			"debug token to find in rc" },
+    "tag",			"debug token to find in rc" },
   { ARGV_LAST }
 };
 
@@ -214,7 +223,7 @@ static	void	choose_shell(void)
   shell = getenv(SHELL_ENVIRON);
   if (shell == NULL) {
     /* oh well, we just guess on c-shell */
-    cshell_b = TRUE;
+    cshell_b = 1;
     return;
   }
   
@@ -228,46 +237,62 @@ static	void	choose_shell(void)
   
   for (shell_c = 0; sh_shells[shell_c] != NULL; shell_c++) {
     if (strcmp(sh_shells[shell_c], shell_p) == 0) {
-      bourne_b = TRUE;
+      bourne_b = 1;
       return;
     }
   }
   
-  cshell_b = TRUE;
+  cshell_b = 1;
 }
 
 /*
  * dump the current flags set in the debug variable VAL
  */
-static	void	dump_debug(const int val)
+static	void	dump_debug(const unsigned long val)
 {
   attr_t	*attr_p;
-  int		tok_c = 0, work = val;
+  char		*str;
+  unsigned long	work = val;
+  int		col_c = 0, len;
   
   for (attr_p = attributes; attr_p->at_string != NULL; attr_p++) {
-    /* the below is necessary to handle the 'none' HACK */
+    /* the below test for work == 0 is necessary to handle the 'none' token */
     if ((work == 0 && attr_p->at_value == 0)
-	|| (attr_p->at_value != 0
-	    && BIT_IS_SET(work, attr_p->at_value))) {
+	|| (attr_p->at_value != 0 && BIT_IS_SET(work, attr_p->at_value))) {
       BIT_CLEAR(work, attr_p->at_value);
       
-      if (tok_c == 0) {
+      if (col_c == 0) {
 	(void)fprintf(stderr, "   ");
+	col_c += 3;
       }
       
       if (very_verbose_b) {
-	(void)fprintf(stderr, "%s -- %s", attr_p->at_string, attr_p->at_desc);
+	(void)fprintf(stderr, "%s (%s) -- %s (%#lx)\n",
+		      attr_p->at_string, attr_p->at_short, attr_p->at_desc,
+		      attr_p->at_value);
+	col_c = 0;
       }
       else {
-	(void)fprintf(stderr, "%s", attr_p->at_string);
+	if (short_tokens_b) {
+	  str = attr_p->at_short;
+	}
+	else {
+	  str = attr_p->at_string;
+	}
+	len = strlen(str);
+	if (col_c + len + 2 > LINE_WIDTH) {
+	  (void)fprintf(stderr, "\n");
+	  (void)fprintf(stderr, "   ");
+	  col_c = 3;
+	}
+	(void)fprintf(stderr, "%s", str);
+	col_c += len;
+	
+	/* if we've got more to go then print the , */
 	if (work != 0) {
 	  (void)fprintf(stderr, ", ");
+	  col_c += 2;
 	}
-	tok_c = (tok_c + 1) % TOKENS_PER_LINE;
-      }
-      
-      if (tok_c == 0) {
-	(void)fprintf(stderr, "\n");
       }
       
       if (work == 0) {
@@ -276,12 +301,12 @@ static	void	dump_debug(const int val)
     }
   }
   
-  if (tok_c != 0) {
+  if (col_c != 0) {
     (void)fprintf(stderr, "\n");
   }
   
   if (work != 0) {
-    (void)fprintf(stderr, "%s: warning, unknown debug flags: %#x\n",
+    (void)fprintf(stderr, "%s: warning, unknown debug flag(s): %#lx\n",
 		  argv_program, work);
   }
 }
@@ -324,7 +349,7 @@ static	long	token_to_value(const char *tok)
 static	int	read_next_token(FILE *infile, long *debug_p,
 				char *token, const int token_size)
 {
-  int	cont_b = FALSE, found_b = FALSE;
+  int	cont_b = 0, found_b = 0;
   long	new_debug = 0;
   char	buf[1024], *tok_p, *buf_p;
   
@@ -337,9 +362,7 @@ static	int	read_next_token(FILE *infile, long *debug_p,
     
     /* chop off the ending \n */
     buf_p = strrchr(buf, '\n');
-    if (buf_p != NULL) {
-      *buf_p = '\0';
-    }
+    SET_POINTER(buf_p, '\0');
     
     buf_p = buf;
     
@@ -364,7 +387,7 @@ static	int	read_next_token(FILE *infile, long *debug_p,
       found_b = 1;
     }
     
-    cont_b = FALSE;
+    cont_b = 0;
     
     while (1) {
       /* get the next token */
@@ -378,7 +401,7 @@ static	int	read_next_token(FILE *infile, long *debug_p,
       
       /* do we have a continuation character? */
       if (strcmp(tok_p, "\\") == 0) {
-	cont_b = TRUE;
+	cont_b = 1;
 	break;
       }
       
@@ -391,9 +414,7 @@ static	int	read_next_token(FILE *infile, long *debug_p,
     }
   }
   
-  if (debug_p != NULL) {
-    *debug_p = new_debug;
-  }
+  SET_POINTER(debug_p, new_debug);
   
   if (found_b) {
     return 1;
@@ -407,22 +428,23 @@ static	int	read_next_token(FILE *infile, long *debug_p,
  * Read in a rc file from PATH and process it looking for the
  * specified DEBUG_VALUE or TAG_FIND token.  It passes back the
  * returned debug value in DEBUG_P.  Passes back the matching TOKEN of
- * TOKEN_SIZE.  Returns 0 if the file was not found.  Returns 1 if the
- * file was found, 2 if the file and the token was found.
+ * TOKEN_SIZE.
+ *
+ * Returns FILE_NOT_FOUND, FILE_FOUND, or TOKEN_FOUND.
  */
 static	int	read_rc_file(const char *path, const long debug_value,
 			     const char *tag_find, long *debug_p,
 			     char *token, const int token_size)
 {
   FILE	*infile;
-  int	found_b = FALSE;
+  int	found_b = 0;
   char	next_token[64];
   long	new_debug;
   
   /* open the path */
   infile = fopen(path, "r");
   if (infile == NULL) {
-    return 0;
+    return FILE_NOT_FOUND;
   }
   
   /* run through the tokens, looking for a match */
@@ -444,19 +466,17 @@ static	int	read_rc_file(const char *path, const long debug_value,
   
   (void)fclose(infile);
   
-  if (debug_p != NULL) {
-    *debug_p = new_debug;
-  }
+  SET_POINTER(debug_p, new_debug);
   if (token != NULL) {
     strncpy(token, next_token, token_size);
     token[token_size - 1] = '\0';
   }
   
   if (found_b) {
-    return 2;
+    return TOKEN_FOUND;
   }
   else {
-    return 1;
+    return FILE_FOUND;
   }
 }
 
@@ -482,13 +502,15 @@ static	long	process(const long debug_value, const char *tag_find,
     ret = read_rc_file(DEFAULT_CONFIG, debug_value, tag_find, &new_debug,
 		       token, token_size);
     /* did we find the correct value in the file? */
-    if (ret == 2) {
+    if (ret == TOKEN_FOUND) {
       return new_debug;
     }
-    path_p = DEFAULT_CONFIG;
     
     /* if we did not find the file, check the home directory */
-    if (ret == 0) {
+    if (ret == FILE_FOUND) {
+      path_p = DEFAULT_CONFIG;
+    }
+    else {
       /* find our home directory */
       home_p = getenv(HOME_ENVIRON);
       if (home_p == NULL) {
@@ -503,10 +525,15 @@ static	long	process(const long debug_value, const char *tag_find,
       ret = read_rc_file(path, debug_value, tag_find, &new_debug,
 			 token, token_size);
       /* did we find the correct value in the file? */
-      if (ret == 2) {
+      if (ret == TOKEN_FOUND) {
 	return new_debug;
       }
-      path_p = path;
+      if (ret == FILE_FOUND) {
+	path_p = path;
+      }
+      else {
+	path_p = NULL;
+      }
     }
   }
   else {
@@ -514,11 +541,11 @@ static	long	process(const long debug_value, const char *tag_find,
     ret = read_rc_file(inpath, debug_value, tag_find, &new_debug,
 		       token, token_size);
     /* did we find the correct value in the file? */
-    if (ret == 2) {
+    if (ret == TOKEN_FOUND) {
       return new_debug;
     }
     /* if the specified was not found, return error */
-    if (ret == 0) {
+    if (ret != FILE_FOUND) {
       (void)fprintf(stderr, "%s: could not read '%s': ",
 		    argv_program, inpath);
       perror("");
@@ -559,8 +586,14 @@ static	long	process(const long debug_value, const char *tag_find,
     
     /* did we not find the token? */
     if (def_p->de_string == NULL) {
-      (void)fprintf(stderr, "%s: could not find tag '%s' in '%s'\n",
-		    argv_program, tag_find, path_p);
+      if (path_p == NULL) {
+	(void)fprintf(stderr, "%s: unknown tag '%s'\n",
+		      argv_program, tag_find);
+      }
+      else {
+	(void)fprintf(stderr, "%s: could not find tag '%s' in '%s'\n",
+		      argv_program, tag_find, path_p);
+      }
       exit(1);
     }
   }
@@ -655,12 +688,13 @@ static	void	list_tags(void)
 static	void	dump_current(void)
 {
   char		*lpath, *start_file, token[64];
-  unsigned long	addr, inter;
+  DMALLOC_PNT	addr;
+  unsigned long	inter;
   long		addr_count;
   int		lock_on, start_line, start_count;
   unsigned int	flags;
   
-  (void)fprintf(stderr, "Debug Malloc Utility: http://www.dmalloc.com/\n");
+  (void)fprintf(stderr, "Debug Malloc Utility: http://dmalloc.com/\n");
   (void)fprintf(stderr,
 		"  For a list of the command-line options enter: %s --usage\n",
 		argv_argv[0]);
@@ -669,7 +703,7 @@ static	void	dump_current(void)
 		       &inter, &lock_on, &lpath,
 		       &start_file, &start_line, &start_count);
   
-  if (flags == (unsigned int)DEBUG_INIT) {
+  if (flags == 0) {
     (void)fprintf(stderr, "Debug-Flags  not-set\n");
   }
   else {
@@ -681,11 +715,11 @@ static	void	dump_current(void)
     }
   }
   
-  if (addr == ADDRESS_INIT) {
+  if (addr == NULL) {
     (void)fprintf(stderr, "Address      not-set\n");
   }
   else {
-    if (addr_count == ADDRESS_COUNT_INIT) {
+    if (addr_count == 0) {
       (void)fprintf(stderr, "Address      %#lx\n", (long)addr);
     }
     else {
@@ -694,31 +728,31 @@ static	void	dump_current(void)
     }
   }
   
-  if (inter == INTERVAL_INIT) {
+  if (inter == 0) {
     (void)fprintf(stderr, "Interval     not-set\n");
   }
   else {
     (void)fprintf(stderr, "Interval     %lu\n", inter);
   }
   
-  if (lock_on == LOCK_ON_INIT) {
+  if (lock_on == 0) {
     (void)fprintf(stderr, "Lock-On      not-set\n");
   }
   else {
     (void)fprintf(stderr, "Lock-On      %d\n", lock_on);
   }
   
-  if (lpath == LOGPATH_INIT) {
+  if (lpath == NULL) {
     (void)fprintf(stderr, "Logpath      not-set\n");
   }
   else {
     (void)fprintf(stderr, "Logpath      '%s'\n", lpath);
   }
   
-  if (start_file == START_FILE_INIT && start_count == START_COUNT_INIT) {
+  if (start_file == NULL && start_count == 0) {
     (void)fprintf(stderr, "Start-File   not-set\n");
   }
-  else if (start_count != START_COUNT_INIT) {
+  else if (start_count > 0) {
     (void)fprintf(stderr, "Start-Count  %d\n", start_count);
   }
   else {
@@ -735,14 +769,14 @@ static	void    set_variable(const char *var, const char *value)
   char	comm[1024];
   
   if (bourne_b) {
-    (void)loc_snprintf(comm, sizeof(comm), "%s=%s;\nexport %s;\n",
+    (void)loc_snprintf(comm, sizeof(comm), "%s=%s\nexport %s\n",
 		       var, value, var);
   }
   else if (gdb_b) {
-    (void)loc_snprintf(comm, sizeof(comm), "set env %s %s;\n", var, value);
+    (void)loc_snprintf(comm, sizeof(comm), "set env %s %s\n", var, value);
   }
   else {
-    (void)loc_snprintf(comm, sizeof(comm), "setenv %s %s;\n", var, value);
+    (void)loc_snprintf(comm, sizeof(comm), "setenv %s %s\n", var, value);
   }
   
   if (make_changes_b) {
@@ -773,21 +807,32 @@ static	char	*local_strerror(const int error_num)
 int	main(int argc, char **argv)
 {
   char		buf[1024];
-  int		debug_set_b = FALSE, set_b = FALSE;
-  char		*lpath = LOGPATH_INIT, *sfile = START_FILE_INIT;
-  unsigned long	addr = ADDRESS_INIT, inter = INTERVAL_INIT;
-  long		addr_count = ADDRESS_COUNT_INIT;
-  int		lock_on = LOCK_ON_INIT;
-  int		sline = START_LINE_INIT, scount = START_COUNT_INIT;
-  unsigned int	flags = DEBUG_INIT;
+  int		set_b = 0;
+  char		*lpath, *sfile;
+  DMALLOC_PNT	addr;
+  unsigned long	inter;
+  long		addr_count;
+  int		lock_on;
+  int		sline, scount;
+  unsigned int	flags;
   
   argv_help_string = "Sets dmalloc library env variables.  Also try --usage.";
   argv_version_string = dmalloc_version;
   
   argv_process(args, argc, argv);
   
+  if (help_b) {
+    (void)fprintf(stderr, "Debug Malloc Utility: http://dmalloc.com/\n");
+    (void)fprintf(stderr,
+		  "  This utility helps set the Debug Malloc environment variables.\n");
+    (void)fprintf(stderr,
+		  "  For a list of the command-line options enter: %s --usage\n",
+		  argv_argv[0]);
+    exit(0);
+  }
+  
   if (very_verbose_b) {
-    verbose_b = TRUE;
+    verbose_b = 1;
   }
   
   /* try to figure out the shell we are using */
@@ -799,135 +844,103 @@ int	main(int argc, char **argv)
   _dmalloc_environ_get(OPTIONS_ENVIRON, &addr, &addr_count, &flags, &inter,
 		       &lock_on, &lpath, &sfile, &sline, &scount);
   
-  /* get a new debug value from tag */
-  if (tag != NULL) {
-    if (debug != NO_VALUE) {
+  /*
+   * So, if a tag was specified on the command line then we set the
+   * debug from it.  If it was not then we see if the debug flags were
+   * set as a hex value from the -d.  If this was not used then take
+   * the current value.
+   */
+  if (tag == NULL) {
+    if (argv_was_used(args, DEBUG_ARG)) {
+      set_b = 1;
+      /* should we clear the rest? */
+      if (remove_auto_b && (! keep_b)) {
+	clear_b = 1;
+      }
+    }
+    else {
+      debug = flags;
+    }
+  }
+  else {
+    if (argv_was_used(args, DEBUG_ARG)) {
       (void)fprintf(stderr, "%s: warning -d ignored, processing tag '%s'\n",
 		    argv_program, tag);
     }
+    set_b = 1;
     debug = process(0L, tag, NULL, 0);
-    debug_set_b = TRUE;
+    /* should we clear the rest? */
+    if (remove_auto_b && (! keep_b)) {
+      clear_b = 1;
+    }
   }
   
   if (plus.aa_entry_n > 0) {
     int		plus_c;
-    
-    /* get current debug value and add tokens if possible */
-    if (debug == NO_VALUE) {
-      if (flags == (unsigned int)DEBUG_INIT) {
-	debug = 0;
-      }
-      else {
-	debug = flags;
-      }
-    }
-    
     for (plus_c = 0; plus_c < plus.aa_entry_n; plus_c++) {
       debug |= token_to_value(ARGV_ARRAY_ENTRY(plus, char *, plus_c));
+      set_b = 1;
     }
   }
   
   if (minus.aa_entry_n > 0) {
     int		minus_c;
-    
-    /* get current debug value and add tokens if possible */
-    if (debug == NO_VALUE) {
-      if (flags == (unsigned int)DEBUG_INIT) {
-	debug = 0;
-      }
-      else {
-	debug = flags;
-      }
-    }
-    
     for (minus_c = 0; minus_c < minus.aa_entry_n; minus_c++) {
       debug &= ~token_to_value(ARGV_ARRAY_ENTRY(minus, char *, minus_c));
+      set_b = 1;
     }
   }
   
-  /*
-   * NOTE: this should be ahead of lock_on setting which tests this.
-   */
-  if (debug != NO_VALUE) {
-    /* special case, undefine if 0 */
-    if (debug == 0) {
-      flags = DEBUG_INIT;
-    }
-    else {
-      flags = debug;
-    }
-    set_b = TRUE;
-    /* should we clear the rest? */
-    if (debug_set_b && remove_auto_b && ! keep_b) {
-      clear_b = TRUE;
-    }
-  }
-  
-  if (clear_b) {
-    set_b = TRUE;
-  }
-  
   if (address != NULL) {
     _dmalloc_address_break(address, &addr, &addr_count);
-    set_b = TRUE;
+    set_b = 1;
   }
   else if (clear_b) {
-    addr = ADDRESS_INIT;
+    addr = NULL;
   }
   
-  if (interval != INTERVAL_NO_VALUE) {
-    /* NOTE: special case, == 0 causes it to be undef'ed */
-    if (interval == 0) {
-      inter = INTERVAL_INIT;
-    }
-    else {
-      inter = interval;
-    }
-    set_b = TRUE;
+  if (argv_was_used(args, INTERVAL_ARG)) {
+    inter = interval;
+    set_b = 1;
   }
   else if (clear_b) {
-    inter = INTERVAL_INIT;
+    inter = 0;
   }
   
   /*
    * NOTE: this should be after the debug setting which this tests.
    */
-  if (thread_lock_on != NO_VALUE) {
-    /* NOTE: special case, == 0 causes it to be undef'ed */
-    if (thread_lock_on == 0) {
-      lock_on = LOCK_ON_INIT;
-    }
-    else {
-      lock_on = thread_lock_on;
-    }
-    set_b = TRUE;
-    if (! BIT_IS_SET(flags, DEBUG_ALLOW_NONLINEAR)) {
+  if (argv_was_used(args, THREAD_LOCK_ON_ARG)) {
+    lock_on = thread_lock_on;
+    set_b = 1;
+    if (BIT_IS_SET(debug, DEBUG_FORCE_LINEAR)) {
       (void)fprintf(stderr,
-		    "WARNING: the allow-nonlinear flag is not enabled\n");
+		    "WARNING: the force-linear flag is enabled\n");
     }
   }
   else if (clear_b) {
-    lock_on = LOCK_ON_INIT;
+    lock_on = 0;
   }
   
   if (logpath != NULL) {
     lpath = logpath;
-    set_b = TRUE;
+    set_b = 1;
   }
   else if (clear_b) {
-    lpath = LOGPATH_INIT;
+    lpath = NULL;
   }
   
   if (start != NULL) {
     _dmalloc_start_break(start, &sfile, &sline, &scount);
-    set_b = TRUE;
+    set_b = 1;
   }
   else if (clear_b) {
-    sfile = START_FILE_INIT;
-    scount = START_COUNT_INIT;
+    sfile = NULL;
+    sline = 0;
+    scount = 0;
   }
   
-  if (errno_to_print != NO_VALUE) {
+  if (errno_to_print > 0) {
     (void)fprintf(stderr, "%s: dmalloc_errno value '%d' = \n",
 		  argv_program, errno_to_print);
     (void)fprintf(stderr, "   '%s'\n", local_strerror(errno_to_print));
@@ -958,7 +971,8 @@ int	main(int argc, char **argv)
       }
       else if (verbose_b) {
 	(void)fprintf(stderr, "%s -- %s\n",
-		      attr_p->at_string, attr_p->at_desc);
+		      (short_tokens_b ? attr_p->at_short : attr_p->at_string),
+		      attr_p->at_desc);
       }
       else {
 	(void)fprintf(stderr, "%s\n", attr_p->at_string);
@@ -967,13 +981,13 @@ int	main(int argc, char **argv)
     }
   }
   
-  if (set_b) {
+  if (clear_b || set_b) {
     _dmalloc_environ_set(buf, sizeof(buf), long_tokens_b, short_tokens_b,
-			 addr, addr_count, flags, inter, lock_on, lpath,
+			 addr, addr_count, debug, inter, lock_on, lpath,
 			 sfile, sline, scount);
     set_variable(OPTIONS_ENVIRON, buf);
   }
-  else if (errno_to_print == NO_VALUE
+  else if (errno_to_print == 0
 	   && (! list_tags_b)
 	   && (! debug_tokens_b)) {
     dump_current();
