@@ -1,7 +1,7 @@
 /*
  * Leap-frog point to allow dmalloc on/off via relink.
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,9 +16,9 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: dmalloc_lp.c,v 1.53 1999/10/13 22:51:06 gray Exp $
+ * $Id: dmalloc_lp.c,v 1.60 2000/11/13 15:47:16 gray Exp $
  */
 
 /*
@@ -75,17 +75,17 @@
 #include "dmalloc_loc.h"
 #include "dmalloc_lp.h"
 #if USE_DMALLOC_LEAP == 0
-#include "malloc.h"				/* for _loc_[m]alloc funcs */
+#include "malloc_funcs.h"			/* for _loc_[m]alloc funcs */
 #include "error.h"				/* for dmalloc_vmessage */
 #endif
 #include "return.h"
 
 #if INCLUDE_RCS_IDS
-#ifdef __GNUC__
-#ident "$Id: dmalloc_lp.c,v 1.53 1999/10/13 22:51:06 gray Exp $";
+#if IDENT_WORKS
+#ident "$Id: dmalloc_lp.c,v 1.60 2000/11/13 15:47:16 gray Exp $"
 #else
 static	char	*rcs_id =
-  "$Id: dmalloc_lp.c,v 1.53 1999/10/13 22:51:06 gray Exp $";
+  "$Id: dmalloc_lp.c,v 1.60 2000/11/13 15:47:16 gray Exp $";
 #endif
 #endif
 
@@ -96,7 +96,7 @@ static	char	*rcs_id =
 int	dmalloc_errno = ERROR_NONE;
 
 /* pre-set dmalloc_debug() value before the library is setup */
-int	_dmalloc_debug_preset = DEBUG_PRE_NONE;
+unsigned int	_dmalloc_debug_preset = DEBUG_PRE_NONE;
 
 #if USE_DMALLOC_LEAP
 /* pointers to shutdown function to allow calls without linked routine */
@@ -138,6 +138,12 @@ dmalloc_vmessage_func_t		_dmalloc_vmessage_func = NULL;
 /* pointers to track function to allow calls without linked routine */
 dmalloc_track_func_t		_dmalloc_track_func = NULL;
 
+/* pointers to track function to allow calls without linked routine */
+dmalloc_mark_func_t		_dmalloc_mark_func = NULL;
+
+/* pointers to track function to allow calls without linked routine */
+dmalloc_log__func_t		_dmalloc_mark_func = NULL;
+
 /* pointers to strerror function to allow calls without linked routine */
 dmalloc_strerror_func_t		_dmalloc_strerror_func = NULL;
 #endif
@@ -592,56 +598,44 @@ void	dmalloc_log_unfreed(void)
  */
 int	dmalloc_verify(const DMALLOC_PNT pnt)
 {
+  char	*file;
+  
+  GET_RET_ADDR(file);
+  
 #if USE_DMALLOC_LEAP
   if (_dmalloc_verify_func == NULL) {
     return DMALLOC_VERIFY_NOERROR;
   }
   else {
-    return _dmalloc_verify_func(pnt);
+    return _dmalloc_verify_func(file, DMALLOC_DEFAULT_LINE, pnt);
   }
 #else
-  return _dmalloc_verify(pnt);
-#endif
-}
-
-/*
- * routine to call malloc_verify when linked in
- */
-int	malloc_verify(const DMALLOC_PNT pnt)
-{
-#if USE_DMALLOC_LEAP
-  if (_dmalloc_verify_func == NULL) {
-    return DMALLOC_VERIFY_NOERROR;
-  }
-  else {
-    return _dmalloc_verify_func(pnt);
-  }
-#else
-  return _dmalloc_verify(pnt);
+  return _dmalloc_verify(file, DMALLOC_DEFAULT_LINE, pnt);
 #endif
 }
 
 /*
  * routine to call dmalloc_debug when linked in
  */
-void	dmalloc_debug(const int flags)
+unsigned int	dmalloc_debug(const unsigned int flags)
 {
 #if USE_DMALLOC_LEAP
   if (_dmalloc_debug_func == NULL) {
     _dmalloc_debug_preset = flags;
+    return 0;
   }
   else {
-    _dmalloc_debug_func(flags);
+    return _dmalloc_debug_func(flags);
   }
 #else
-  _dmalloc_debug(flags);
+  return _dmalloc_debug(flags);
 #endif
 }
 
 /*
  * routine to call dmalloc_debug_current when linked in
  */
-int	dmalloc_debug_current(void)
+unsigned int	dmalloc_debug_current(void)
 {
 #if USE_DMALLOC_LEAP
   if (_dmalloc_debug_current_func == NULL) {
@@ -728,6 +722,44 @@ void	dmalloc_track(const dmalloc_track_t track_func)
 #endif
 }
 
+/*
+ * routine to call dmalloc_mark when linked in
+ */
+unsigned long	dmalloc_mark(void)
+{
+#if USE_DMALLOC_LEAP
+  if (_dmalloc_mark_func != NULL) {
+    return _dmalloc_mark_func();
+  }
+  else {
+    return 0;
+  }
+#else
+  return _dmalloc_mark();
+#endif
+}
+
+/*
+ * routine to call dmalloc_log_changed when linked in
+ */
+void	dmalloc_log_changed(const unsigned long mark, const int not_freed_b,
+			    const int free_b, const int details_b)
+{
+  char	*file;
+  
+  GET_RET_ADDR(file);
+  
+#if USE_DMALLOC_LEAP
+  if (_dmalloc_mark_func != NULL) {
+    _dmalloc_log_changed(file, DMALLOC_DEFAULT_LINE, mark, not_freed_b,
+			 free_b, details_b);
+  }
+#else
+  _dmalloc_log_changed(file, DMALLOC_DEFAULT_LINE, mark, not_freed_b,
+		       free_b, details_b);
+#endif
+}
+
 /*
  * routine to call dmalloc_strerror when linked in
  */
