@@ -1,7 +1,7 @@
 /*
  * Test program for malloc code
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,9 +16,9 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $
+ * $Id: dmalloc_t.c,v 1.91 2000/11/13 15:46:52 gray Exp $
  */
 
 /*
@@ -56,11 +56,11 @@
 #include "error_val.h"
 
 #if INCLUDE_RCS_IDS
-#ifdef __GNUC__
-#ident "$Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $";
+#if IDENT_WORKS
+#ident "$Id: dmalloc_t.c,v 1.91 2000/11/13 15:46:52 gray Exp $"
 #else
 static	char	*rcs_id =
-  "$Id: dmalloc_t.c,v 1.82 1999/03/10 23:16:13 gray Exp $";
+  "$Id: dmalloc_t.c,v 1.91 2000/11/13 15:46:52 gray Exp $";
 #endif
 #endif
 
@@ -265,11 +265,11 @@ static	int	do_random(const int iter_n)
       }
 #endif
       
-      if (flags & DEBUG_ALLOW_NONLINEAR) {
-	which = random_value(16);
+      if (flags & DEBUG_FORCE_LINEAR) {
+	which = random_value(15);
       }
       else {
-	which = random_value(15);
+	which = random_value(16);
       }
       
       switch (which) {
@@ -379,7 +379,11 @@ static	int	do_random(const int iter_n)
 	
       case 15:
 #if HAVE_SBRK
-	{
+	/*
+	 * random-debug and sbrk kills the library because the
+	 * DEBUG_ALLOW_NONLINEAR flag can be turned off.
+	 */
+	if (! random_debug_b) {
 	  void	*mem;
 	  
 	  mem = sbrk(amount);
@@ -526,7 +530,7 @@ static	int	check_special(void)
   }
   free(NULL);
 #if ALLOW_FREE_NULL
-  if (dmalloc_errno != ERROR_NONE && ! silent_b) {
+  if (dmalloc_errno != ERROR_NONE && (! silent_b)) {
     (void)printf("   ERROR: free of 0L returned error.\n");
   }
 #else
@@ -610,7 +614,9 @@ static	void	do_interactive(void)
       
       (void)printf("\tmap       - map the heap to the logfile\n");
       (void)printf("\tstats     - dump heap stats to the logfile\n");
-      (void)printf("\tunfreed   - list the unfree memory to the logfile\n\n");
+      (void)printf("\tunfreed   - list the unfree memory to the logfile\n");
+      (void)printf("\tmark      - display the current mark value\n");
+      (void)printf("\tchanged   - display what pointers have changed\n\n");
       
       (void)printf("\tverify    - check out a memory address (or all heap)\n");
       (void)printf("\toverwrite - overwrite some memory to test errors\n");
@@ -751,6 +757,21 @@ static	void	do_interactive(void)
       continue;
     }
     
+    if (strncmp(line, "mark", len) == 0) {
+      (void)printf("Mark is %lu.\n", dmalloc_mark());
+      continue;
+    }
+    
+    if (strncmp(line, "changed", len) == 0) {
+      (void)printf("Enter the mark: ");
+      if (fgets(line, sizeof(line), stdin) == NULL) {
+	break;
+      }
+      dmalloc_log_changed(atoi(line), 1, 1, 1);
+      (void)printf("Done.\n");
+      continue;
+    }
+    
     if (strncmp(line, "overwrite", len) == 0) {
       char	*overwrite = "OVERWRITTEN";
       
@@ -820,7 +841,7 @@ static	void	do_interactive(void)
       ret = malloc_verify((char *)pnt);
       (void)printf("malloc_verify(%#lx) returned '%s'\n",
 		   (long)pnt,
-		   (ret == DMALLOC_VERIFY_NOERROR ? "success" : "failure"));
+		   (ret == DMALLOC_NOERROR ? "success" : "failure"));
       continue;
     }
     
@@ -989,7 +1010,7 @@ int	main(int argc, char **argv)
   
   /* last thing is to verify the heap */
   ret = malloc_verify(NULL);
-  if (ret != DMALLOC_VERIFY_NOERROR) {
+  if (ret != DMALLOC_NOERROR) {
     (void)fprintf(stderr, "Final malloc_verify returned failure: %s (%d)\n",
 		  dmalloc_strerror(dmalloc_errno), dmalloc_errno);
   }
