@@ -1,7 +1,7 @@
 /*
  * Memory table routines
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,16 +16,16 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: dmalloc_tab.c,v 1.9 1999/03/11 22:26:03 gray Exp $
+ * $Id: dmalloc_tab.c,v 1.15 2000/11/13 15:46:52 gray Exp $
  */
 
 /*
  * This file contains routines used to maintain and display a memory
  * table by file and line number of memory usage.
  *
- * Inspired by code from PSM <psm @ sics.se>.  Thanks much.
+ * Inspired by code from PSM.  Thanks much.
  */
 
 #if HAVE_STDLIB_H
@@ -47,11 +47,11 @@
 #include "dmalloc_tab_loc.h"
 
 #if INCLUDE_RCS_IDS
-#ifdef __GNUC__
-#ident "$Id: dmalloc_tab.c,v 1.9 1999/03/11 22:26:03 gray Exp $";
+#if IDENT_WORKS
+#ident "$Id: dmalloc_tab.c,v 1.15 2000/11/13 15:46:52 gray Exp $"
 #else
 static	char	*rcs_id =
-  "$Id: dmalloc_tab.c,v 1.9 1999/03/11 22:26:03 gray Exp $";
+  "$Id: dmalloc_tab.c,v 1.15 2000/11/13 15:46:52 gray Exp $";
 #endif
 #endif
 
@@ -76,9 +76,8 @@ static	mem_table_t	memory_table[MEM_ENTRIES_N];
  * example, if you need only 10 bits, do h = (h & hashmask(10)); In
  * which case, the hash table should have hashsize(10) elements.
  *
- * By Bob Jenkins, 1996.  bob_jenkins@compuserve.com.  You may use
- * this code any way you wish, private, educational, or commercial.
- * It's free.  See
+ * By Bob Jenkins, 1996.  You may use this code any way you wish,
+ * private, educational, or commercial.  It's free.  See
  * http://ourworld.compuserve.com/homepages/bob_jenkins/evahash.htm
  * Use for hash table lookup, or anything where one collision in 2^^32
  * is acceptable.  Do NOT use for cryptographic purposes.
@@ -755,7 +754,7 @@ void	_table_log_info(const int entry_n, const int in_use_b)
   
   /* is the table empty */
   if (table_entry_c == 0) {
-    _dmalloc_message("  memory table is empty");
+    _dmalloc_message(" memory table is empty");
     return;
   }
   
