@@ -1,7 +1,7 @@
 /*
  * user-level memory-allocation routines
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,9 +16,9 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: malloc.c,v 1.138 1999/10/13 22:39:31 gray Exp $
+ * $Id: malloc.c,v 1.150 2000/11/13 15:46:31 gray Exp $
  */
 
 /*
@@ -75,64 +75,47 @@
 #include "heap.h"
 #include "dmalloc_loc.h"
 #include "dmalloc_lp.h"
+#include "malloc_funcs.h"
 #include "return.h"
 
 #if INCLUDE_RCS_IDS
-#ifdef __GNUC__
-#ident "$Id: malloc.c,v 1.138 1999/10/13 22:39:31 gray Exp $";
+#if IDENT_WORKS
+#ident "$Id: malloc.c,v 1.150 2000/11/13 15:46:31 gray Exp $"
 #else
 static	char	*rcs_id =
-  "$Id: malloc.c,v 1.138 1999/10/13 22:39:31 gray Exp $";
+  "$Id: malloc.c,v 1.150 2000/11/13 15:46:31 gray Exp $";
 #endif
 #endif
 
-/* local routines */
-static	int	dmalloc_startup(void);
-void		_dmalloc_shutdown(void);
-DMALLOC_PNT	_loc_malloc(const char *file, const int line,
-			    const DMALLOC_SIZE size, const int func_id,
-			    const DMALLOC_SIZE alignment);
-DMALLOC_PNT	_loc_realloc(const char *file, const int line,
-			     DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size,
-			     const int func_id);
-int	_loc_free(const char *file, const int line, DMALLOC_PNT pnt);
-void	_dmalloc_log_heap_map(const char *file, const int line);
-void	_dmalloc_log_stats(const char *file, const int line);
-void	_dmalloc_log_unfreed(const char *file, const int line);
-int	_dmalloc_verify(const DMALLOC_PNT pnt);
-void	_dmalloc_debug(const int flags);
-int	_dmalloc_debug_current(void);
-int	_dmalloc_examine(const char *file, const int line,
-			 const DMALLOC_PNT pnt, DMALLOC_SIZE *size_p,
-			 char **file_p, unsigned int *line_p,
-			 DMALLOC_PNT *ret_attr_p);
-void	_dmalloc_track(const dmalloc_track_t track_func);
-const char	*_dmalloc_strerror(const int error_num);
-
 /* local variables */
-static	int		enabled_b = FALSE;	/* have we started yet? */
-static	int		in_alloc_b = FALSE;	/* can't be here twice */
-static	int		do_shutdown_b = FALSE;	/* execute shutdown soon */
-static	int		memalign_warn_b = FALSE; /* memalign warning printed?*/
+static	int		enabled_b = 0;		/* have we started yet? */
+static	int		in_alloc_b = 0;		/* can't be here twice */
+static	int		do_shutdown_b = 0;	/* execute shutdown soon */
+static	int		memalign_warn_b = 0;	/* memalign warning printed?*/
 static	dmalloc_track_t	tracking_func = NULL;	/* memory trxn tracking func */
 
 /* debug variables */
-static	char		*start_file = START_FILE_INIT;	/* file to start at */
-static	int		start_line = START_LINE_INIT;	/* line to start */
-static	int		start_count = START_COUNT_INIT; /* start after X */
-static	int		thread_lock_on = LOCK_ON_INIT;	/* start locking when*/
-static	int		thread_lock_c = 0;		/* lock counter */
+static	char		*start_file = NULL;	/* file to start at */
+static	int		start_line = 0;		/* line to start */
+static	int		start_count = 0;	/* start after X */
+static	int		thread_lock_c = 0;	/* lock counter */
 
 /****************************** thread locking *******************************/
 
 #if LOCK_THREADS
 #ifdef THREAD_MUTEX_T
-/* define a global variable we use as a lock counter */
-#ifdef PTHREAD_MUTEX_INITIALIZER
-static THREAD_MUTEX_T dmalloc_mutex = PTHREAD_MUTEX_INITIALIZER;
-#else
+/*
+ * Define a global variable we use as a lock counter.
+ *
+ * NOTE: we do not use the PTHREAD_MUTEX_INITIALIZER since this
+ * basically delays the pthread_mutex_init call to when
+ * pthread_mutex_lock is called for the first time (at least on
+ * freebsd).  Since we don't want to go recursive into the pthread
+ * library when we go to lock our mutex variable, we want to force the
+ * initialization to happen beforehand with a call to
+ * pthread_mute_init.
+ */
 static THREAD_MUTEX_T dmalloc_mutex;
-#endif
 #else
 #error We need to have THREAD_MUTEX_T defined by the configure script
 #endif
@@ -203,8 +186,16 @@ static	void	unlock_thread(void)
      * into the library.  Ugh.
      */
     if (thread_lock_c == THREAD_INIT_LOCK) {
-#if HAVE_PTHREAD_MUTEX_INIT && !defined(PTHREAD_MUTEX_INITIALIZER)
-      /* we init instead of the lock point to avoid recursion */
+#if HAVE_PTHREAD_MUTEX_INIT
+      /*
+       * NOTE: we do not use the PTHREAD_MUTEX_INITIALIZER since this
+       * basically delays the pthread_mutex_init call to when
+       * pthread_mutex_lock is called for the first time (at least on
+       * freebsd).  Since we don't want to go recursive into the
+       * pthread library when we go to lock our mutex variable, we
+       * want to force the initialization to happen beforehand with a
+       * call to pthread_mute_init.
+       */
       pthread_mutex_init(&dmalloc_mutex, THREAD_LOCK_INIT_VAL);
 #endif
     }
@@ -219,76 +210,6 @@ static	void	unlock_thread(void)
 
 /****************************** local utilities ******************************/
 
-/*
- * a call to the alloc routines has been made, check the debug variables
- * returns ERROR or NOERROR.
- */
-static	int	check_debug_vars(const char *file, const int line)
-{
-  if (_dmalloc_aborting_b) {
-    return ERROR;
-  }
-  
-  /*
-   * NOTE: we need to do this outside of lock to get env vars
-   * otherwise our thread_lock_on variable won't be initialized and
-   * the THREAD_LOCK will flip.
-   */
-  if (! enabled_b) {
-    if (dmalloc_startup() != NOERROR) {
-      return ERROR;
-    }
-  }
-  
-#if LOCK_THREADS
-  lock_thread();
-#endif
-  
-  if (in_alloc_b) {
-    dmalloc_errno = ERROR_IN_TWICE;
-    dmalloc_error("check_debug_vars");
-    /* NOTE: dmalloc_error may die already */
-    _dmalloc_die(FALSE);
-    /*NOTREACHED*/
-  }
-  
-  in_alloc_b = TRUE;
-  
-  /* check start file/line specifications */
-  if (! BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)
-      && start_file != START_FILE_INIT
-      && file != DMALLOC_DEFAULT_FILE
-      && line != DMALLOC_DEFAULT_LINE
-      && strcmp(start_file, file) == 0
-      && (start_line == 0 || start_line == line)) {
-    BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
-  }
-  
-  /* start checking heap after X times */
-  if (start_count != START_COUNT_INIT && --start_count == 0) {
-    BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
-  }
-  
-  /* checking heap every X times */
-  _dmalloc_iter_c++;
-  if (_dmalloc_check_interval != INTERVAL_INIT
-      && _dmalloc_check_interval > 0) {
-    if (_dmalloc_iter_c % _dmalloc_check_interval == 0) {
-      BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
-    }
-    else { 
-      BIT_CLEAR(_dmalloc_flags, DEBUG_CHECK_HEAP);
-    }
-  }
-  
-  /* after all that, do we need to check the heap? */
-  if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)) {
-    (void)_chunk_check();
-  }
-  
-  return NOERROR;
-}
-
 /*
  * check out a pointer to see if we were looking for it.  this should
  * be re-entrant and it may not return.
@@ -299,7 +220,7 @@ static	void	check_pnt(const char *file, const int line, const void *pnt,
   static unsigned long	addr_c = 0;
   char			where_buf[64];
   
-  if (_dmalloc_address == ADDRESS_INIT || pnt != _dmalloc_address) {
+  if (_dmalloc_address == NULL || pnt != _dmalloc_address) {
     return;
   }
   
@@ -309,9 +230,8 @@ static	void	check_pnt(const char *file, const int line, const void *pnt,
 		   _chunk_desc_pnt(where_buf, sizeof(where_buf), file, line));
   
   /* NOTE: if address_seen_n == 0 then never quit */
-  if (_dmalloc_address_seen_n == ADDRESS_COUNT_INIT
-      || (_dmalloc_address_seen_n > 0
-	  && addr_c >= (unsigned long)_dmalloc_address_seen_n)) {
+  if (_dmalloc_address_seen_n > 0
+      && addr_c >= _dmalloc_address_seen_n) {
     dmalloc_errno = ERROR_IS_FOUND;
     dmalloc_error("check_pnt");
   }
@@ -322,17 +242,12 @@ static	void	check_pnt(const char *file, const int line, const void *pnt,
  */
 static	void	process_environ(void)
 {
-  _dmalloc_environ_get(OPTIONS_ENVIRON, (unsigned long *)&_dmalloc_address,
+  _dmalloc_environ_get(OPTIONS_ENVIRON, &_dmalloc_address,
 		       &_dmalloc_address_seen_n, &_dmalloc_flags,
-		       &_dmalloc_check_interval, &thread_lock_on,
+		       &_dmalloc_check_interval, &_dmalloc_lock_on,
 		       &_dmalloc_logpath, &start_file, &start_line,
 		       &start_count);
-  thread_lock_c = thread_lock_on;
-  
-  /* if it was not set then no flags set */
-  if (_dmalloc_flags == (unsigned int)DEBUG_INIT) {
-    _dmalloc_flags = 0;
-  }
+  thread_lock_c = _dmalloc_lock_on;
   
   /* if we set the start stuff, then check-heap comes on later */
   if (start_count != -1) {
@@ -346,7 +261,7 @@ static	void	process_environ(void)
   
 #if LOCK_THREADS == 0
   /* was thread-lock-on specified but not configured? */
-  if (thread_lock_on != LOCK_ON_INIT) {
+  if (_dmalloc_lock_on > 0) {
     dmalloc_errno = ERROR_LOCK_NOT_CONFIG;
     _dmalloc_die(0);
   }
@@ -364,7 +279,7 @@ static	RETSIGTYPE	signal_handler(const int sig)
   _dmalloc_message("caught signal %d", sig);
   /* if we are already inside malloc then do the shutdown later */
   if (in_alloc_b) {
-    do_shutdown_b = TRUE;
+    do_shutdown_b = 1;
   }
   else {
     _dmalloc_shutdown();
@@ -377,52 +292,72 @@ static	RETSIGTYPE	signal_handler(const int sig)
  */
 static	int	dmalloc_startup(void)
 {
+  static int	some_up_b = 0;
+  
   /* have we started already? */
   if (enabled_b) {
-    return ERROR;
+    return 0;
   }
   
-  /* set this here so if an error occurs below, it will not try again */
-  enabled_b = TRUE;
-  
+  if (! some_up_b) {
+    /* set this up here so if an error occurs below, it will not try again */
+    some_up_b = 1;
+    
 #if STORE_TIMEVAL
-  GET_TIMEVAL(_dmalloc_start);
+    GET_TIMEVAL(_dmalloc_start);
 #else
 #if HAVE_TIME /* NOT STORE_TIME */
-  _dmalloc_start = time(NULL);
+    _dmalloc_start = time(NULL);
 #endif
 #endif
-  
-  /* process the environmental variable(s) */
-  process_environ();
-  
-  /* startup heap code */
-  if (_heap_startup() == ERROR) {
-    return ERROR;
-  }
-  
-  /* startup the chunk lower-level code */
-  if (_chunk_startup() == ERROR) {
-    return ERROR;
-  }
-  
-  /* set leap variables */
+    
+    /* process the environmental variable(s) */
+    process_environ();
+    
+    /* startup heap code */
+    if (! _heap_startup()) {
+      return 0;
+    }
+    
+    /* startup the chunk lower-level code */
+    if (! _chunk_startup()) {
+      return 0;
+    }
+    
+    /* set leap variables */
 #if USE_DMALLOC_LEAP
-  _dmalloc_malloc_func = _loc_malloc;
-  _dmalloc_realloc_func = _loc_realloc;
-  _dmalloc_free_func = _loc_free;
-  _dmalloc_shutdown_func = _dmalloc_shutdown;
-  _dmalloc_log_heap_map_func = _dmalloc_log_heap_map;
-  _dmalloc_log_stats_func = _dmalloc_log_stats;
-  _dmalloc_log_unfreed_func = _dmalloc_log_unfreed;
-  _dmalloc_verify_func = _dmalloc_verify;
-  _dmalloc_debug_func = _dmalloc_debug;
-  _dmalloc_debug_current_func = _dmalloc_debug_current;
-  _dmalloc_examine_func = _dmalloc_examine;
-  _dmalloc_vmessage_func = _dmalloc_vmessage;
-  _dmalloc_track_func = _dmalloc_track;
-  _dmalloc_strerror_func = _dmalloc_strerror;
+    _dmalloc_malloc_func = _loc_malloc;
+    _dmalloc_realloc_func = _loc_realloc;
+    _dmalloc_free_func = _loc_free;
+    _dmalloc_shutdown_func = _dmalloc_shutdown;
+    _dmalloc_log_heap_map_func = _dmalloc_log_heap_map;
+    _dmalloc_log_stats_func = _dmalloc_log_stats;
+    _dmalloc_log_unfreed_func = _dmalloc_log_unfreed;
+    _dmalloc_verify_func = _dmalloc_verify;
+    _dmalloc_debug_func = _dmalloc_debug;
+    _dmalloc_debug_current_func = _dmalloc_debug_current;
+    _dmalloc_examine_func = _dmalloc_examine;
+    _dmalloc_vmessage_func = _dmalloc_vmessage;
+    _dmalloc_track_func = _dmalloc_track;
+    _dmalloc_mark_func = _dmalloc_mark;
+    _dmalloc_log_changed_func = _dmalloc_log_changed;
+    _dmalloc_strerror_func = _dmalloc_strerror;
 #endif
+  }
+  
+#if LOCK_THREADS
+  if (thread_lock_c > 0) {
+    return 1;
+  }
+#endif
+  
+  /*
+   * We have initialized all of our code.
+   *
+   * NOTE: set this up here so if an error occurs below, it will not
+   * try again
+   */
+  enabled_b = 1;
   
   /*
    * NOTE: we may go recursive below here becasue atexit or on_exit
@@ -464,12 +399,11 @@ static	int	dmalloc_startup(void)
   }
 #endif /* SIGNAL_OKAY */
   
-  return NOERROR;
+  return 1;
 }
 
 /*
- * shutdown memory-allocation module, provide statistics if necessary
- * NOTE: called by way of leap routine in dmalloc_lp.c
+ * Shutdown memory-allocation module, provide statistics if necessary
  */
 void	_dmalloc_shutdown(void)
 {
@@ -480,6 +414,13 @@ void	_dmalloc_shutdown(void)
     return;
   }
   
+  /*
+   * Make sure that the log file is open.  We do this here because we
+   * might cause an allocation in the open() and don't want to go
+   * recursive.
+   */
+  _dmalloc_open_log();
+  
 #if LOCK_THREADS
   lock_thread();
 #endif
@@ -489,7 +430,7 @@ void	_dmalloc_shutdown(void)
     return;
   }
   
-  in_alloc_b = TRUE;
+  in_alloc_b = 1;
   
   /* check the heap since we are dumping info from it */
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)) {
@@ -506,7 +447,13 @@ void	_dmalloc_shutdown(void)
   
   /* report on non-freed pointers */
   if (BIT_IS_SET(_dmalloc_flags, DEBUG_LOG_NONFREE)) {
-    _chunk_dump_unfreed();
+    _chunk_log_changed(0, 1, 0,
+#if DUMP_UNFREED_SUMMARY_ONLY
+		       0
+#else
+		       1
+#endif
+		       );
   }
   
 #if STORE_TIMEVAL
@@ -531,7 +478,7 @@ void	_dmalloc_shutdown(void)
 #endif
 #endif
   
-  in_alloc_b = FALSE;
+  in_alloc_b = 0;
   
 #if LOCK_THREADS
   unlock_thread();
@@ -553,12 +500,102 @@ void	__fini_dmalloc()
 }
 #endif
 
+/*
+ * Call to the alloc routines has been made.  Do some initialization,
+ * locking, and check some debug variables.
+ *
+ * Returns ERROR or NOERROR.
+ */
+static	int	dmalloc_in(const char *file, const int line,
+			   const int check_heap_b)
+{
+  if (_dmalloc_aborting_b) {
+    return 0;
+  }
+  
+  /*
+   * NOTE: we need to do this outside of lock to get env vars
+   * otherwise our _dmalloc_lock_on variable won't be initialized and
+   * the THREAD_LOCK will flip.
+   */
+  if (! enabled_b) {
+    if (! dmalloc_startup()) {
+      return 0;
+    }
+  }
+  
+#if LOCK_THREADS
+  lock_thread();
+#endif
+  
+  if (in_alloc_b) {
+    dmalloc_errno = ERROR_IN_TWICE;
+    dmalloc_error("dmalloc_in");
+    /* NOTE: dmalloc_error may die already */
+    _dmalloc_die(0);
+    /*NOTREACHED*/
+  }
+  
+  in_alloc_b = 1;
+  
+  /* check start file/line specifications */
+  if ((! BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP))
+      && start_file != NULL
+      && file != DMALLOC_DEFAULT_FILE
+      && line != DMALLOC_DEFAULT_LINE
+      && strcmp(start_file, file) == 0
+      && (start_line == 0 || start_line == line)) {
+    BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
+  }
+  
+  /* start checking heap after X times */
+  if (start_count > 0 && --start_count == 0) {
+    BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
+  }
+  
+  /* checking heap every X times */
+  _dmalloc_iter_c++;
+  if (_dmalloc_check_interval > 0) {
+    if (_dmalloc_iter_c % _dmalloc_check_interval == 0) {
+      BIT_SET(_dmalloc_flags, DEBUG_CHECK_HEAP);
+    }
+    else { 
+      BIT_CLEAR(_dmalloc_flags, DEBUG_CHECK_HEAP);
+    }
+  }
+  
+  /* after all that, do we need to check the heap? */
+  if (check_heap_b && BIT_IS_SET(_dmalloc_flags, DEBUG_CHECK_HEAP)) {
+    (void)_chunk_check();
+  }
+  
+  return 1;
+}
+
+/*
+ * Going out of the alloc routines back to user space.
+ */
+static	void	dmalloc_out(void)
+{
+  in_alloc_b = 0;
+  
+#if LOCK_THREADS
+  unlock_thread();
+#endif
+  
+  if (do_shutdown_b) {
+    _dmalloc_shutdown();
+  }
+}
+
 /******************************* memory calls ********************************/
 
 /*
  * Allocate and return a SIZE block of bytes.  FUNC_ID contains the
  * type of function.  If we are aligning our malloc then ALIGNMENT is
- * greater than 0.  Returns 0L on error.
+ * greater than 0.
+ *
+ * Returns 0L on error.
  */
 DMALLOC_PNT	_loc_malloc(const char *file, const int line,
 			    const DMALLOC_SIZE size, const int func_id,
@@ -578,7 +615,7 @@ DMALLOC_PNT	_loc_malloc(const char *file, const int line,
   }
 #endif
   
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     if (tracking_func != NULL) {
       tracking_func(file, line, func_id, size, alignment, NULL, NULL);
     }
@@ -607,17 +644,10 @@ DMALLOC_PNT	_loc_malloc(const char *file, const int line,
   }
   
   new_p = _chunk_malloc(file, line, size, func_id, align);
-  in_alloc_b = FALSE;
   
   check_pnt(file, line, new_p, "malloc");
   
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
   
   if (tracking_func != NULL) {
     tracking_func(file, line, func_id, size, alignment, NULL, new_p);
@@ -632,7 +662,9 @@ DMALLOC_PNT	_loc_malloc(const char *file, const int line,
  * OLD_PNT is 0L then it will do the equivalent of malloc(NEW_SIZE).
  * If NEW_SIZE is 0 and OLD_PNT is not 0L then it will do the
  * equivalent of free(OLD_PNT) and will return 0L.  If the RECALLOC_B
- * flag is enabled, it will zero any new memory.  Returns 0L on error.
+ * flag is enabled, it will zero any new memory.
+ *
+ * Returns 0L on error.
  */
 DMALLOC_PNT	_loc_realloc(const char *file, const int line,
 			     DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size,
@@ -651,7 +683,7 @@ DMALLOC_PNT	_loc_realloc(const char *file, const int line,
   }
 #endif
   
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     if (tracking_func != NULL) {
       tracking_func(file, line, func_id, new_size, 0, old_pnt, NULL);
     }
@@ -677,9 +709,9 @@ DMALLOC_PNT	_loc_realloc(const char *file, const int line,
 #if ALLOW_REALLOC_SIZE_ZERO
     if (new_size == 0) {
       /*
-       * If realloc(old_pnt, 0) then free(old_pnt).  Thanks to
-       * Stefan.Froehlich@tuwien.ac.at for patiently pointing that the
-       * realloc in just about every Unix has this functionality.
+       * If realloc(old_pnt, 0) then free(old_pnt).  Thanks to Stefan
+       * Froehlich for patiently pointing that the realloc in just
+       * about every Unix has this functionality.
        */
       (void)_chunk_free(file, line, old_pnt, 0);
       new_p = NULL;
@@ -688,19 +720,11 @@ DMALLOC_PNT	_loc_realloc(const char *file, const int line,
 #endif
       new_p = _chunk_realloc(file, line, old_pnt, new_size, func_id);
   
-  in_alloc_b = FALSE;
-  
   if (new_p != NULL) {
     check_pnt(file, line, new_p, "realloc-out");
   }
   
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
   
   if (tracking_func != NULL) {
     tracking_func(file, line, func_id, new_size, 0, old_pnt, new_p);
@@ -710,13 +734,15 @@ DMALLOC_PNT	_loc_realloc(const char *file, const int line,
 }
 
 /*
- * release PNT in the heap, returning FREE_ERROR, FREE_NOERROR.
+ * Release PNT in the heap.
+ *
+ * Returns FREE_ERROR, FREE_NOERROR.
  */
 int	_loc_free(const char *file, const int line, DMALLOC_PNT pnt)
 {
   int		ret;
   
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     if (tracking_func != NULL) {
       tracking_func(file, line, DMALLOC_FUNC_FREE, 0, 0, pnt, NULL);
     }
@@ -727,15 +753,7 @@ int	_loc_free(const char *file, const int line, DMALLOC_PNT pnt)
   
   ret = _chunk_free(file, line, pnt, 0);
   
-  in_alloc_b = FALSE;
-  
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
   
   if (tracking_func != NULL) {
     tracking_func(file, line, DMALLOC_FUNC_FREE, 0, 0, pnt, NULL);
@@ -747,7 +765,9 @@ int	_loc_free(const char *file, const int line, DMALLOC_PNT pnt)
 /*************************** external memory calls ***************************/
 
 /*
- * allocate and return a SIZE block of bytes.  returns 0L on error.
+ * Allocate and return a SIZE block of bytes.
+ *
+ * Returns 0L on error.
  */
 #undef malloc
 DMALLOC_PNT	malloc(DMALLOC_SIZE size)
@@ -759,9 +779,10 @@ DMALLOC_PNT	malloc(DMALLOC_SIZE size)
 }
 
 /*
- * allocate and return a block of _zeroed_ bytes able to hold
- * NUM_ELEMENTS, each element contains SIZE bytes.  returns 0L on
- * error.
+ * Allocate and return a block of _zeroed_ bytes able to hold
+ * NUM_ELEMENTS, each element contains SIZE bytes.
+ *
+ * Returns 0L on error.
  */
 #undef calloc
 DMALLOC_PNT	calloc(DMALLOC_SIZE num_elements, DMALLOC_SIZE size)
@@ -778,8 +799,9 @@ DMALLOC_PNT	calloc(DMALLOC_SIZE num_elements, DMALLOC_SIZE size)
  * either copying all of OLD_PNT to the new area or truncating.  If
  * OLD_PNT is 0L then it will do the equivalent of malloc(NEW_SIZE).
  * If NEW_SIZE is 0 and OLD_PNT is not 0L then it will do the
- * equivalent of free(OLD_PNT) and will return 0L.  Returns 0L on
- * error.
+ * equivalent of free(OLD_PNT) and will return 0L.
+ *
+ * Returns 0L on error.
  */
 #undef realloc
 DMALLOC_PNT	realloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
@@ -797,7 +819,9 @@ DMALLOC_PNT	realloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
  * OLD_PNT is 0L then it will do the equivalent of malloc(NEW_SIZE).
  * If NEW_SIZE is 0 and OLD_PNT is not 0L then it will do the
  * equivalent of free(OLD_PNT) and will return 0L.  Any extended
- * memory space will be zeroed like calloc.  Returns 0L on error.
+ * memory space will be zeroed like calloc.
+ *
+ * Returns 0L on error.
  */
 #undef recalloc
 DMALLOC_PNT	recalloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
@@ -812,7 +836,9 @@ DMALLOC_PNT	recalloc(DMALLOC_PNT old_pnt, DMALLOC_SIZE new_size)
 /*
  * Allocate and return a SIZE block of bytes that has been aligned to
  * ALIGNMENT bytes.  ALIGNMENT must be a power of two and must be less
- * than or equal to the block-size.  Returns 0L on error.
+ * than or equal to the block-size.
+ *
+ * Returns 0L on error.
  */
 #undef memalign
 DMALLOC_PNT	memalign(DMALLOC_SIZE alignment, DMALLOC_SIZE size)
@@ -826,7 +852,9 @@ DMALLOC_PNT	memalign(DMALLOC_SIZE alignment, DMALLOC_SIZE size)
 
 /*
  * Allocate and return a SIZE block of bytes that has been aligned to
- * a page-size.  Returns 0L on error.
+ * a page-size.
+ *
+ * Returns 0L on error.
  */
 #undef valloc
 DMALLOC_PNT	valloc(DMALLOC_SIZE size)
@@ -841,7 +869,9 @@ DMALLOC_PNT	valloc(DMALLOC_SIZE size)
 #ifndef DMALLOC_STRDUP_MACRO
 /*
  * Allocate and return a block of bytes that contains the string STR
- * including the \0.  Returns 0L on error.
+ * including the \0.
+ *
+ * Returns 0L on error.
  */
 #undef strdup
 char	*strdup(const char *str)
@@ -865,8 +895,10 @@ char	*strdup(const char *str)
 #endif
 
 /*
- * release PNT in the heap, returning FREE_ERROR, FREE_NOERROR or void
- * depending on whether STDC is defined by your compiler.
+ * Release PNT in the heap.
+ *
+ * Returns FREE_ERROR, FREE_NOERROR or void depending on whether STDC
+ * is defined by your compiler.
  */
 #undef free
 DMALLOC_FREE_RET	free(DMALLOC_PNT pnt)
@@ -904,60 +936,41 @@ DMALLOC_FREE_RET	cfree(DMALLOC_PNT pnt)
 /******************************* utility calls *******************************/
 
 /*
- * log the heap structure plus information on the blocks if necessary.
- * NOTE: called by way of leap routine in dmalloc_lp.c
+ * Log the heap structure plus information on the blocks if necessary.
  */
 void	_dmalloc_log_heap_map(const char *file, const int line)
 {
   /* check the heap since we are dumping info from it */
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     return;
   }
   
   _chunk_log_heap_map();
   
-  in_alloc_b = FALSE;
-  
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
 }
 
 /*
- * dump dmalloc statistics to logfile
- * NOTE: called by way of leap routine in dmalloc_lp.c
+ * Dump dmalloc statistics to logfile.
  */
 void	_dmalloc_log_stats(const char *file, const int line)
 {
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     return;
   }
   
   _chunk_list_count();
   _chunk_stats();
   
-  in_alloc_b = FALSE;
-  
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
 }
 
 /*
- * dump unfreed-memory info to logfile
- * NOTE: called by way of leap routine in dmalloc_lp.c
+ * Dump unfreed-memory info to logfile.
  */
 void	_dmalloc_log_unfreed(const char *file, const int line)
 {
-  if (check_debug_vars(file, line) != NOERROR) {
+  if (! dmalloc_in(file, line, 1)) {
     return;
   }
   
@@ -965,48 +978,37 @@ void	_dmalloc_log_unfreed(const char *file, const int line)
     _dmalloc_message("dumping the unfreed pointers");
   }
   
-  _chunk_dump_unfreed();
-  
-  in_alloc_b = FALSE;
-  
-#if LOCK_THREADS
-  unlock_thread();
+  /*
+   * to log the non-free we are interested in the pointers currently
+   * being used
+   */
+  _chunk_log_changed(0, 1, 0,
+#if DUMP_UNFREED_SUMMARY_ONLY
+		     0
+#else
+		     1
 #endif
+		     );
   
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
+  dmalloc_out();
 }
 
 /*
- * verify pointer PNT, if PNT is 0 then check the entire heap.
- * NOTE: called by way of leap routine in dmalloc_lp.c
- * returns MALLOC_VERIFY_ERROR or MALLOC_VERIFY_NOERROR
+ * Verify pointer PNT, if PNT is 0 then check the entire heap.
+ *
+ * Returns MALLOC_VERIFY_ERROR or MALLOC_VERIFY_NOERROR
  */
-int	_dmalloc_verify(const DMALLOC_PNT pnt)
+int	_dmalloc_verify(const char *file, const int line,
+			const DMALLOC_PNT pnt)
 {
   int	ret;
   
+  if (! dmalloc_in(file, line, 0)) {
+    return MALLOC_VERIFY_NOERROR;
+  }
+  
   /* should not check heap here because we will be doing it below */
   
-  if (_dmalloc_aborting_b) {
-    return DMALLOC_VERIFY_ERROR;
-  }
-  
-#if LOCK_THREADS
-  lock_thread();
-#endif
-  
-  if (in_alloc_b) {
-    dmalloc_errno = ERROR_IN_TWICE;
-    dmalloc_error("dmalloc_verify");
-    /* NOTE: dmalloc_error may die already */
-    _dmalloc_die(FALSE);
-    /*NOTREACHED*/
-  }
-  
-  in_alloc_b = TRUE;
-  
   if (pnt == NULL) {
     ret = _chunk_check();
   }
@@ -1014,61 +1016,109 @@ int	_dmalloc_verify(const DMALLOC_PNT pnt)
     ret = _chunk_pnt_check("dmalloc_verify", pnt, CHUNK_PNT_EXACT, 0);
   }
   
-  in_alloc_b = FALSE;
+  dmalloc_out();
   
-#if LOCK_THREADS
-  unlock_thread();
-#endif
-  
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
-  
-  if (ret == NOERROR) {
-    return DMALLOC_VERIFY_NOERROR;
+  if (ret) {
+    return MALLOC_VERIFY_NOERROR;
   }
   else {
-    return DMALLOC_VERIFY_ERROR;
+    return MALLOC_VERIFY_ERROR;
   }
 }
 
 /*
- * set the global debug functionality FLAGS (0 to disable all
- * debugging).  NOTE: after this module has started up, you cannot set
- * certain flags such as fence-post or free-space checking.
+ * Verify pointer PNT, if PNT is 0 then check the entire heap.
+ *
+ * Returns MALLOC_VERIFY_ERROR or MALLOC_VERIFY_NOERROR
  */
-void	_dmalloc_debug(const int flags)
+int	malloc_verify(const DMALLOC_PNT pnt)
 {
-  /* should not check the heap here since we are setting the debug variable */
+  char	*file;
   
-  /* if we've not started up then set the variable */
-  if (! enabled_b) {
-    _dmalloc_flags = flags;
-  }
-  else {
-    /* make sure that the not-changeable flag values are preserved */
-    _dmalloc_flags &= DEBUG_NOT_CHANGEABLE;
-    
-    /* add the new flags - the not-addable ones */
-    _dmalloc_flags |= flags & ~DEBUG_NOT_ADDABLE;
-  }
+  GET_RET_ADDR(file);
+  
+  return _dmalloc_verify(file, DMALLOC_DEFAULT_LINE, pnt);
 }
 
 /*
- * returns the current debug functionality flags.  this allows you to
+ * Set the global debug functionality FLAGS (0 to disable all
+ * debugging).
+ *
+ * NOTE: you cannot remove certain flags such as signal handlers since
+ * they are setup at initialization time only.  Also you cannot add
+ * certain flags such as fence-post or free-space checking since they
+ * must be on from the start.
+ *
+ * Returns the old debug flag value.
+ */
+unsigned int	_dmalloc_debug(const unsigned int flags)
+{
+  unsigned int	old_flags;
+  
+  if (! enabled_b) {
+    (void)dmalloc_startup();
+  }
+  
+  old_flags = _dmalloc_flags;
+  
+  /* make sure that the not-removable flags are preserved */
+  _dmalloc_flags &= DEBUG_NOT_REMOVABLE;
+  
+  /* add the new flags - the not-addable ones */
+  _dmalloc_flags |= flags & ~DEBUG_NOT_ADDABLE;
+  
+  return old_flags;
+}
+
+/*
+ * Returns the current debug functionality flags.  This allows you to
  * save a dmalloc library state to be restored later.
  */
-int	_dmalloc_debug_current(void)
+unsigned int	_dmalloc_debug_current(void)
 {
+  if (! enabled_b) {
+    (void)dmalloc_startup();
+  }
+  
   /* should not check the heap here since we are dumping the debug variable */
   return _dmalloc_flags;
 }
 
 /*
- * examine pointer PNT and returns SIZE, and FILE / LINE info on it,
- * or return-address RET_ADDR if any of the pointers are not 0L.
- * if FILE returns 0L then RET_ATTR may have a value and vice versa.
- * returns NOERROR or ERROR depending on whether PNT is good or not
+ * int _dmalloc_examine
+ *
+ * DESCRIPTION:
+ *
+ * Examine a pointer and return information on its allocation size as
+ * well as the file and line-number where it was allocated.  If the
+ * file and line number is not available, then it will return the
+ * allocation location's return-address if available.
+ *
+ * RETURNS:
+ *
+ * Success - DMALLOC_NOERROR
+ *
+ * Failure - DMALLOC_ERROR
+ *
+ * ARGUMENTS:
+ *
+ * file -> File were we are examining the pointer.
+ *
+ * line -> Line-number from where we are examining the pointer.
+ *
+ * pnt -> Pointer we are checking.
+ *
+ * size_p <- Pointer to an unsigned int which, if not NULL, will be
+ * set to the size of bytes from the pointer.
+ *
+ * file_p <- Pointer to a character pointer which, if not NULL, will
+ * be set to the file where the pointer was allocated.
+ *
+ * line_p <- Pointer to a character pointer which, if not NULL, will
+ * be set to the line-number where the pointer was allocated.
+ *
+ * ret_attr_p <- Pointer to a void pointer, if not NULL, will be set
+ * to the return-address where the pointer was allocated.
  */
 int	_dmalloc_examine(const char *file, const int line,
 			 const DMALLOC_PNT pnt, DMALLOC_SIZE *size_p,
@@ -1079,31 +1129,24 @@ int	_dmalloc_examine(const char *file, const int line,
   unsigned int	size_map;
   
   /* need to check the heap here since we are geting info from it below */
-  if (check_debug_vars(file, line) != NOERROR) {
-    return ERROR;
+  if (! dmalloc_in(file, line, 1)) {
+    return DMALLOC_ERROR;
   }
   
   /* NOTE: we do not need the alloc-size info */
-  ret = _chunk_read_info(pnt, &size_map, NULL, file_p, line_p, ret_attr_p,
-			 "dmalloc_examine", NULL, NULL);
-  in_alloc_b = FALSE;
+  ret = _chunk_read_info(pnt, "dmalloc_examine", &size_map, NULL, file_p,
+			 line_p, ret_attr_p, NULL, NULL);
   
-#if LOCK_THREADS
-  unlock_thread();
-#endif
+  dmalloc_out();
   
-  if (do_shutdown_b) {
-    _dmalloc_shutdown();
-  }
-  
-  if (ret == NOERROR) {
+  if (ret) {
     if (size_p != NULL) {
       *size_p = size_map;
     }
-    return NOERROR;
+    return DMALLOC_NOERROR;
   }
   else {
-    return ERROR;
+    return DMALLOC_ERROR;
   }
 }
 
@@ -1113,22 +1156,55 @@ int	_dmalloc_examine(const char *file, const int line,
  */
 void	_dmalloc_track(const dmalloc_track_t track_func)
 {
-  if (! enabled_b) {
-    (void)dmalloc_startup();
-  }
   tracking_func = track_func;
 }
 
+/*
+ * Return to the caller the current ``mark'' which can be used later
+ * to dmalloc_log_changed pointers since this point.  Multiple marks
+ * can be saved and used.
+ */
+unsigned long	_dmalloc_mark(void)
+{
+  if (! enabled_b) {
+    (void)dmalloc_startup();
+  }
+  
+  return _dmalloc_iter_c;
+}
+
+/*
+ * Dump the pointers that have changed since the mark which was
+ * returned by dmalloc_mark.  If not_freed_b is set to non-0 then log
+ * the new pointers that are non-freed.  If free_b is set to non-0
+ * then log the new pointers that are freed.  If details_b set to
+ * non-0 then dump the individual pointers that have changed otherwise
+ * just dump the summaries.
+ */
+void	_dmalloc_log_changed(const char *file, const int line,
+			     const unsigned long mark, const int not_freed_b,
+			     const int free_b, const int details_b)
+{
+  if (! dmalloc_in(file, line, 1)) {
+    return;
+  }
+  
+  _chunk_log_changed(mark, not_freed_b, free_b, details_b);
+  
+  dmalloc_out();
+}
+
 /*
  * Dmalloc version of strerror to return the string version of
- * ERROR_NUM.  Returns an invaid errno string if ERROR_NUM is
- * out-of-range.
+ * ERROR_NUM.
+ *
+ * Returns an invaid errno string if ERROR_NUM is out-of-range.
  */
 const char	*_dmalloc_strerror(const int error_num)
 {
   error_str_t	*err_p;
   
-  /* should not check_debug_vars here because _dmalloc_error calls this */
+  /* should not dmalloc_in here because _dmalloc_error calls this */
   
   for (err_p = error_list; err_p->es_error != 0; err_p++) {
     if (err_p->es_error == error_num) {
