@@ -1,7 +1,7 @@
 /*
  * version string for the library
  *
- * Copyright 1999 by Gray Watson
+ * Copyright 2000 by Gray Watson
  *
  * This file is part of the dmalloc package.
  *
@@ -16,9 +16,9 @@
  * software described herein for any purpose.  It is provided "as is"
  * without express or implied warranty.
  *
- * The author may be contacted via http://www.dmalloc.com/
+ * The author may be contacted via http://dmalloc.com/
  *
- * $Id: version.h,v 1.89 1999/10/18 16:12:18 gray Exp $
+ * $Id: version.h,v 1.106.2.1 2001/02/26 17:18:55 gray Exp $
  */
 
 #ifndef __VERSION_H__
@@ -29,15 +29,17 @@
  * should be entered in:
  *
  *	Changlog
- *	RELEASE.html
  *	NEWS
+ *	RELEASE.html
+ *	Makefile.bsd.ports
+ *	dmalloc.spec
  *	dmalloc.texi (2 places)
- *	dmalloc.h.3 (defines at top of file)
+ *	dmalloc.h.3 (DMALLOC_VERSION defines at top of file)
  *
  * Make sure to also cvs tag the release.  dmalloc_release_X_X_X[_bX]
  */
-static	char	*dmalloc_version = "4.3.0";
+static	char	*dmalloc_version = "4.8.2" ;
 
-/* Version Date: $Date: 1999/10/18 16:12:18 $ */
+/* Version Date: $Date: 2001/02/26 17:18:55 $ */
 
 #endif /* ! __VERSION_H__ */
